<?php
/**
 * Automation Hook action class for MRM Autoamtion
 *
 * Class Webhook
 *
 * @package MailMintPro\Mint\Internal
 */

namespace MailMintPro\Mint\Internal;

use MailMintPro\Mint\Internal\Traits\Singleton;
use MintMail\App\Internal\Automation\HelperFunctions;
use MintMailPro\Mint_Pro_Helper;
/**
 * Automation Hook action class for MRM Autoamtion
 *
 * Class Webhook
 *
 * @package MailMintPro\Mint\Internal\AutomationHooks
 */
class AutomationHooks {

	use Singleton;

	/**
	 * MainMint filter hook.
	 *
	 * @return void
	 */
	public function init() {
		add_filter( 'mint_supported_automation_actions', array( $this, 'mint_pro_automation_actions' ), 10, 1 );
		add_filter( 'mrm_automation_connectors', array( $this, 'mint_pro_automation_connectors' ), 10, 1 );
		add_action( 'mailmint_process_customer_win_back', array( $this, 'mint_schedule_customer_win_back_trigger' ), 10, 2 );
	}

	/**
	 * Add new action.
	 *
	 * @param array $actions Get action for automaiton.
	 * @return mixed
	 */
	public function mint_pro_automation_actions( $actions ) {
		$actions['createUser']        = 'Create user';
		$actions['createUserMeta']    = 'Create user meta';
		$actions['updateStatus']      = 'Update status';
		$actions['removeTag']         = 'Remove tag';
		$actions['addList']           = 'Add list';
		$actions['removeList']        = 'Remove list';
		$actions['condition']         = 'Condition';
		$actions['createCoupon']      = 'Create coupon';
		$actions['addContactNote']    = 'Add contact note';
		$actions['sequence']          = 'Email sequence';
		$actions['webHookOutgoing']   = 'Web Hook';
		$actions['specificTimeDelay'] = 'Specific Time Delay';
		$actions['stopAutomation']    = 'Stop Automation';
		$actions['deleteCoupon']      = 'Delete Coupon';

		return $actions;
	}

	/**
	 * Add new connector.
	 *
	 * @param array $connectors Get Connector.
	 * @return mixed
	 */
	public function mint_pro_automation_connectors( $connectors ) {
		if ( HelperFunctions::is_wc_active() ) {
			$connectors['woocommerce'] = array(
				'class_name' => 'ConnectorWoocommerce',
			);
		}

		if ( HelperFunctions::is_wpf_active() ) {
			$connectors['wpfunnels'] = array(
				'class_name' => 'ConnectorWpfunnels',
			);
		}

		if ( Mint_Pro_Helper::is_edd_active() ) {
			$connectors['edd'] = array(
				'class_name' => 'ConnectorEdd',
			);
		}

		if ( Mint_Pro_Helper::is_gform_active() ) {
			$connectors['gform'] = array(
				'class_name' => 'ConnectorGform',
			);
		}

		if ( Mint_Pro_Helper::is_tutor_active() ) {
			$connectors['tutor'] = array(
				'class_name' => 'ConnectorTutor',
			);
		}

		if ( Mint_Pro_Helper::is_jetform_active() ) {
			$connectors['jetform'] = array(
				'class_name' => 'ConnectorJetform',
			);
		}

		if ( Mint_Pro_Helper::is_fluentform_active() ) {
			$connectors['fluentforms'] = array(
				'class_name' => 'ConnectorFluentForm',
			);
		}

		if ( Mint_Pro_Helper::is_contact_form_7_active() ) {
			$connectors['contactform'] = array(
				'class_name' => 'ConnectorContactForm',
			);
		}

		if ( Mint_Pro_Helper::is_learndash_lms_active() ) {
			$connectors['learndash'] = array(
				'class_name' => 'ConnectorLearnDash',
			);
		}

		if ( Mint_Pro_Helper::is_memberpress_active() ) {
			$connectors['memberpress'] = array(
				'class_name' => 'ConnectorMemberPress',
			);
		}
		return $connectors;
	}

	/**
	 * Summary: Schedules customer win-back trigger for automation.
	 *
	 * Description: This method schedules the customer win-back trigger for automation based on the provided steps and automation ID.
	 * It extracts relevant information from the trigger settings, calculates the starting timestamp and interval for the recurring action, and schedules the action using the Asynchronous Scheduler library.
	 *
	 * @access public
	 *
	 * @param array $steps An array containing information about the customer win-back steps, including the trigger.
	 * @param int   $automation_id The ID of the automation associated with the customer win-back trigger.
	 *
	 * @return void
	 *
	 * @since 1.7.1
	 */
	public function mint_schedule_customer_win_back_trigger( $steps, $automation_id ) {
		// Extract trigger information.
		$trigger = !empty( $steps ) ? $steps[0] : array();
		$step_id = isset( $trigger['step_id'] ) ? $trigger['step_id'] : '';

		// Extract settings from the trigger.
		$settings = isset( $trigger['settings']['wc_customer_winback_settings'] ) ? $trigger['settings']['wc_customer_winback_settings'] : array();

		$start_date  = isset( $settings['start_on'] ) ? $settings['start_on'] : '';
		$start_time  = isset( $settings['run_at'] ) ? $settings['run_at'] : '';
		$schedule_at = $start_date . ' ' . $start_time;

		$recurring_every = isset( $settings['automation_frequency'] ) ? $settings['automation_frequency'] : '';

		// Calculate the starting timestamp for the recurring action.
		$timestamp = $this->get_recurring_starting_timestamp( $schedule_at );
		$interval  = $recurring_every * DAY_IN_SECONDS;

		// Prepare arguments for the recurring action callback.
		$args = array(
			array(
				'automation_id' => $automation_id,
				'step_id'       => $step_id,
				'settings'      => $settings,
			),
		);

		// Schedule recurring action using the Asynchronous Scheduler library.
		$group = 'mailmint-customer-winback-schedule-' . $automation_id;
		as_schedule_recurring_action( $timestamp, $interval, MAILMINT_PROCESS_CUSTOMER_WINBACK_AUTOMATION, $args, $group );
	}

	/**
	 * Summary: Calculate the starting timestamp for a recurring action.
	 *
	 * Description: This method calculates the starting timestamp for a recurring action based on the provided schedule date.
	 *
	 * @access public
	 *
	 * @param string $schedule_date The scheduled date for the recurring action in 'Y-m-d H:i:s' format.
	 *
	 * @return int Returns the starting timestamp for the recurring action.
	 *
	 * @since 1.7.1
	 */
	public function get_recurring_starting_timestamp( $schedule_date ) {
		$current_date  = new \DateTime( 'now', wp_timezone() );
		$schedule_date = date_create( gmdate( 'Y-m-d H:i:s', strtotime( $schedule_date ) ), wp_timezone() );
		$date_diff     = date_diff( $schedule_date, $current_date );
		$date_diff     = '+' . $date_diff->y . 'year' . $date_diff->m . 'month' . $date_diff->d . 'day' . $date_diff->h . 'hour' . $date_diff->i . 'minute' . ( $date_diff->s + 1 ) . 'second';
		return strtotime( $date_diff );
	}

}
